<?php
class JobManager extends Module
{
	
	//returns false if timed-out, true otherwise: 
	public function runJobs( $threadHandlerURL, $jobs, $jobHandler, $waitUntilFinished, $jobTimeoutSeconds = 600) {
		
		$db = loadModule('Database');
		$workerCacheDAO = $db->getDAO('WorkerCacheDAO');

		//reset workers:
		$workerCacheDAO->resetAllWorkers();

		$getWorkerTimeout = 60;

		$workerFailureLimit = 3;
		$workerFailures = 0;
		//loop over all jobs, but remove them as we go.
		while( $job = array_shift( $jobs) ) {

			if( $worker = $workerCacheDAO->getWorker( true, $getWorkerTimeout) ) {
				
				$workerData = array(
					'worker_id' => $worker['wrkr_id'],
					'worker_handler' => $jobHandler,
					'worker_handler_data' => $job,
				);
				$workerDataJSON = json_encode( $workerData);
				
				//base64 to escape command, but try to find better escape method.
				$workerDataJSON64 = base64_encode( $workerDataJSON);

				$workerURL = $threadHandlerURL . '?data=' . $workerDataJSON64; 

				$command = 'curl ' . $workerURL . ' > /dev/null 2>&1 &';
				//logMessage('command: ' . $command);
				exec( $command);

			} else {
				//no worker available/timeout
				$workerFailures++;
			}

			if( $workerFailures >= $workerFailureLimit) {
				logVar("runJobs failed to find a worker after $workerFailureLimit attempts");
				return false;
			}

		}
		
		//dont return until all jobs have finished:
		if( $waitUntilFinished) {
			
			$sleepDuration = 10;
			$scriptTimeExtension = $sleepDuration + 20;
			$secondsWaited = 0;
			
			while( $secondsWaited < $jobTimeoutSeconds) {

				//extend script execution time:
				set_time_limit( $scriptTimeExtension);

				sleep( $sleepDuration);	//wait 10 seconds to see if all finished.
				if( $this->allJobsFinished() ) {
					//logMessage('All jobs finished.');
					return true;
				}

				$secondsWaited += $sleepDuration;
			}

			//time out:
			logMessage('Oh no! Job timed-out');
			return false;
		}
		
		return true;
	}
	
	public function allJobsFinished() {
		$db = loadModule('Database');
		$workerCacheDAO = $db->getDAO('WorkerCacheDAO');
		
		$WorkersInUseCount = $workerCacheDAO->getInUseCount();
		
		if( $WorkersInUseCount > 0) {
			return false;
		} else {
			//wait a moment to see if still finished (not new workers have started):
			//if there are a fair few workers, this should be pretty unlikely because if there are more jobs,
			//they should have been allocated to a worker already (before ALL workers finish)
			sleep( 4);
			$WorkersInUseCount = $workerCacheDAO->getInUseCount();
			if( $WorkersInUseCount > 0) {
				//so another worker has started:
				return false;
			}

		}

		return true;
	}
	
	public function runWorker() {

		$workerDataJSON64 = requestParam('data');
		$workerDataJSON = base64_decode( $workerDataJSON64);
		$workerData = json_decode( $workerDataJSON, true);
		
		$workerId = $workerData['worker_id'];
		$threadHandler = $workerData['worker_handler'];	//name of 'ThreadHandler' class to actually process the tracker.
		$threadHandlerData = $workerData['worker_handler_data'];
		
		try {
			$threadJobHandler = $this->loadThreadJobHandler( $threadHandler);
			if( !empty( $threadJobHandler) ) {
				$response = $threadJobHandler->run( $threadHandlerData);
			}  else{
				logMessage('Error - thread job hander empty');
			}

		}
		catch( Exception $e) {
			//gulp!
		}

		//logMessage( 'setAvailable: ' . $workerId);
		$db = loadModule('Database');
		$workerCacheDAO = $db->getDAO('WorkerCacheDAO');
		$workerCacheDAO->setAvailable( $workerId);
	}
	
	public function loadThreadJobHandler( $threadHandlerName) {

		//look for a FemtooTrackerJobHandler.php file and 
		//create and return new instance of FemtooTrackerJobHandler
		
		global $SITE_LOCATION;
		global $CONTENTIGO_HOME;

		$actionFileName = $SITE_LOCATION . '/thread_job_handlers/' . $threadHandlerName . '.php';
		if( !file_exists( $actionFileName) ) {
			return null;
		}
		
		//great
		require_once( $CONTENTIGO_HOME . '/modules/JobManager/ThreadJobHander.php');
		require_once( $actionFileName);
		$threadJobHandler = new $threadHandlerName;

		return $threadJobHandler;
	}
	
}
?>